@@ -7,9 +7,11 @@ class Setup:
     INIT_SCRIPT_PATH = 'jal_init.sql'
     UPDATES_PATH = 'updates'
     ICONS_PATH = "img"
+    IMPORT_PATH = "data_import"
+    IMPORT_SCHEMA_NAME = "import_schema.json"
     LANG_PATH = "languages"
     UPDATE_PREFIX = 'jal_delta_'
-    TARGET_SCHEMA = 25
+    TARGET_SCHEMA = 27
     CALC_TOLERANCE = 1e-10
     DISP_TOLERANCE = 1e-4
 
