@@ -1,66 +1,22 @@
 import json
+from jsonschema import validate
+from jsonschema.exceptions import ValidationError
 import sys
+import os
 import logging
-from datetime import datetime
+from datetime import datetime, timezone
+from collections import defaultdict
 
+from PySide2.QtWidgets import QDialog, QMessageBox
 from jal.widgets.helpers import g_tr
 from jal.constants import Setup, MarketDataFeed, PredefinedAsset, DividendSubtype, CorporateAction
-from jal.db.helpers import account_last_date
+from jal.db.helpers import account_last_date, get_app_path
 from jal.db.update import JalDB
-if "pytest" not in sys.modules:
-    from PySide2.QtCore import Slot
-    from PySide2.QtWidgets import QApplication, QDialog, QMessageBox
-    from jal.ui.ui_select_account_dlg import Ui_SelectAccountDlg
-
-
-# -----------------------------------------------------------------------------------------------------------------------
-# FIXME - adopt if for better usage with pytest framework
-if "pytest" not in sys.modules:
-    class SelectAccountDialog(QDialog, Ui_SelectAccountDlg):
-        def __init__(self, description, current_account, recent_account=None):
-            QDialog.__init__(self)
-            self.setupUi(self)
-            self.account_id = recent_account
-            self.current_account = current_account
-
-            self.DescriptionLbl.setText(description)
-            if self.account_id:
-                self.AccountWidget.selected_id = self.account_id
-
-            # center dialog with respect to main application window
-            parent = None
-            for widget in QApplication.topLevelWidgets():
-                if widget.objectName() == Setup.MAIN_WND_NAME:
-                    parent = widget
-            if parent:
-                x = parent.x() + parent.width() / 2 - self.width() / 2
-                y = parent.y() + parent.height() / 2 - self.height() / 2
-                self.setGeometry(x, y, self.width(), self.height())
-
-        @Slot()
-        def closeEvent(self, event):
-            self.account_id = self.AccountWidget.selected_id
-            if self.AccountWidget.selected_id == 0:
-                QMessageBox().warning(None, g_tr('ReferenceDataDialog', "No selection"),
-                                      g_tr('ReferenceDataDialog', "Invalid account selected"),
-                                      QMessageBox.Ok)
-                event.ignore()
-                return
-
-            if self.AccountWidget.selected_id == self.current_account:
-                QMessageBox().warning(None, g_tr('ReferenceDataDialog', "No selection"),
-                                      g_tr('ReferenceDataDialog', "Please select different account"),
-                                      QMessageBox.Ok)
-                event.ignore()
-                return
-
-            self.setResult(QDialog.Accepted)
-            event.accept()
+from jal.widgets.account_select import SelectAccountDialog
 
 
 class FOF:
     PERIOD = "period"
-
     ACCOUNTS = "accounts"
     ASSETS = "assets"
     TRADES = "trades"
@@ -87,6 +43,19 @@ class FOF:
     PAYMENT_DIVIDEND = "dividend"
     PAYMENT_INTEREST = "interest"
 
+    def __init__(self):
+        pass
+
+    @staticmethod
+    def convert_predefined_asset_type(asset_type):
+        asset_types = {
+            PredefinedAsset.Stock: FOF.ASSET_STOCK,
+            PredefinedAsset.Bond: FOF.ASSET_BOND,
+            PredefinedAsset.ETF: FOF.ASSET_ETF,
+            PredefinedAsset.Derivative: FOF.ASSET_FUTURES
+        }
+        return asset_types[asset_type]
+
 
 class Statement_ImportError(Exception):
     pass
@@ -117,12 +86,35 @@ class Statement:
         'NASDAQ': MarketDataFeed.US,
         'TSE': MarketDataFeed.CA,
         'SBF': MarketDataFeed.EU,
-        'AMEX': MarketDataFeed.US
+        'AMEX': MarketDataFeed.US,
+        'MOEX': MarketDataFeed.RU
     }
     
     def __init__(self):
         self._data = {}
         self._last_selected_account = None
+        self._section_loaders = {
+            FOF.PERIOD: self._check_period,
+            FOF.ASSETS: self._import_assets,
+            FOF.ACCOUNTS: self._import_accounts,
+            FOF.INCOME_SPENDING: self._import_imcomes_and_spendings,
+            FOF.TRANSFERS: self._import_transfers,
+            FOF.TRADES: self._import_trades,
+            FOF.ASSET_PAYMENTS: self._import_asset_payments,
+            FOF.CORP_ACTIONS: self._import_corporate_actions
+        }
+
+    # returns tuple (start_timestamp, end_timestamp)
+    def period(self):
+        if FOF.PERIOD in self._data:
+            return self._data[FOF.PERIOD][0], self._data[FOF.PERIOD][1]
+        else:
+            return 0, 0
+
+    # returns timestamp that is equal to the last second of initial timestamp
+    def _end_of_date(self, timestamp) -> int:
+        end_of_day = datetime.utcfromtimestamp(timestamp).replace(hour=23, minute=59, second=59)
+        return int(end_of_day.replace(tzinfo=timezone.utc).timestamp())
 
     # Loads JSON statement format from file defined by 'filename'
     def load(self, filename: str) -> None:
@@ -134,7 +126,12 @@ class Statement:
                 except json.JSONDecodeError:
                     logging.error(g_tr('Statement', "Failed to read JSON from file: ") + filename)
         except Exception as err:
-            logging.error(g_tr('Statement', "Failed to read file: ") + str(err))
+            raise Statement_ImportError(g_tr('Statement', "Failed to read file: ") + str(err))
+        unsupported_sections = [x for x in self._data if x not in self._section_loaders]
+        if unsupported_sections:
+            for section in unsupported_sections:
+                self._data.pop(section)
+            logging.warning(g_tr("Statement", "Some sections are not supported: ") + unsupported_sections)
 
     # check are assets and accounts from self._data present in database
     # replace IDs in self._data with IDs from database (DB IDs will be negative, initial IDs will be positive)
@@ -147,9 +144,13 @@ class Statement:
         for asset in self._data[FOF.ASSETS]:
             isin = asset['isin'] if 'isin' in asset else ''
             reg_code = asset['reg_code'] if 'reg_code' in asset else ''
-            name = asset['name'] if 'name' in asset else ''     # FIXME: update isin/reg_code/country in DB if different
-            asset_id = JalDB().get_asset_id(asset['symbol'], isin=isin, reg_code=reg_code, name=name, dialog_new=verbal)
-            if asset_id:
+            name = asset['name'] if 'name' in asset else ''
+            country_code = asset['country'] if 'country' in asset else ''
+            expiry = asset['expiry'] if 'expiry' in asset else 0
+            asset_id = JalDB().get_asset_id(asset['symbol'], isin=isin, reg_code=reg_code, name=name, expiry=expiry,
+                                            dialog_new=verbal)
+            if asset_id is not None:
+                JalDB().update_asset_data(asset_id, asset['symbol'], isin, reg_code, country_code)
                 old_id, asset['id'] = asset['id'], -asset_id
                 self._update_id("asset", old_id, asset_id)
                 if asset['type'] == FOF.ASSET_MONEY:
@@ -163,45 +164,47 @@ class Statement:
                 old_id, account['id'] = account['id'], -account_id
                 self._update_id("account", old_id, account_id)
 
-    # Replace 'old_value' with 'new_value' in keys 'tag_name' of self._data
+    # Replace 'old_value' with 'new_value' in keys 'tag_name' of sections listed in mutable_sections
     def _update_id(self, tag_name, old_value, new_value):
-        for section in self._data:
-            if type(self._data[section]) != list:
-                continue
-            if section == FOF.PERIOD:
-                continue   # FIXME Here should be a list of supported sections instead of not supported
+        mutable_sections = [FOF.ACCOUNTS, FOF.ASSETS, FOF.TRADES, FOF.TRANSFERS, FOF.CORP_ACTIONS, FOF.ASSET_PAYMENTS,
+                            FOF.INCOME_SPENDING]
+        for section in mutable_sections:
             for element in self._data[section]:
                 for tag in element:
                     if tag == tag_name:
                         if type(element[tag]) == list:
-                            element[tag] = [-new_value if x==old_value else x for x in element[tag]]
+                            element[tag] = [-new_value if x == old_value else x for x in element[tag]]
                         else:
                             element[tag] = -new_value if element[tag] == old_value else element[tag]
 
-    def import_into_db(self):
-        sections = {
-            FOF.PERIOD: self._check_period,
-            FOF.ASSETS: self._import_assets,
-            FOF.ACCOUNTS: self._import_accounts,
-            FOF.INCOME_SPENDING: self._import_imcomes_and_spendings,
-            FOF.TRANSFERS: self._import_transfers,
-            FOF.TRADES: self._import_trades,
-            FOF.ASSET_PAYMENTS: self._import_asset_payments,
-            FOF.CORP_ACTIONS: self._import_corporate_actions
-        }
-        
-        for section in sections:
-            if section in self._data and sections[section]:
-                sections[section](self._data[section])
-        for section in self._data:
-            if section not in sections:
-                logging.warning(g_tr("Statement", "Section is not supported: ") + section)  # FIXME here should be a list of sections to load
+    def validate_format(self):
+        schema_name = get_app_path() + Setup.IMPORT_PATH + os.sep + Setup.IMPORT_SCHEMA_NAME
+        try:
+            with open(schema_name, 'r') as schema_file:
+                try:
+                    statement_schema = json.load(schema_file)
+                except json.JSONDecodeError:
+                    raise Statement_ImportError(g_tr('Statement', "Failed to read JSON schema from: ") + schema_name)
+        except Exception as err:
+            raise Statement_ImportError(g_tr('Statement', "Failed to read file: ") + str(err))
+        try:
+            validate(instance=self._data, schema=statement_schema)
+        except ValidationError:
+            raise Statement_ImportError(g_tr('StatementLoader', "Statement validation failed"))
 
-        # FIXME This display should be outside of this method
+    # Store content of JSON statement into database
+    # Returns a dict of dict with amounts:
+    # { account_1: { asset_1: X, asset_2: Y, ...}, account_2: { asset_N: Z, ...}, ... }
+    def import_into_db(self):
+        for section in self._section_loaders:
+            if section in self._data:
+                self._section_loaders[section](self._data[section])
+
+        totals = defaultdict(dict)
         for account in self._data[FOF.ACCOUNTS]:
             if 'cash_end' in account:
-                logging.info(g_tr('Statement', 'Planned cash: ') + f"{account['cash_end']:.2f} " +
-                             f"{JalDB().get_asset_name(JalDB().get_account_currency(-account['currency']))}")
+                totals[-account['id']][-account['currency']] = account['cash_end']
+        return totals
 
     def _check_period(self, period):
         if len(period) != 2:
@@ -227,11 +230,12 @@ class Statement:
             reg_code = asset['reg_code'] if 'reg_code' in asset else ''
             name = asset['name'] if 'name' in asset else ''
             country_code = asset['country'] if 'country' in asset else ''
+            expiry = asset['expiry'] if 'expiry' in asset else 0
             try:
                 source = self._sources[asset['exchange']]
             except KeyError:
                 source = MarketDataFeed.NA
-            asset_id = JalDB().add_asset(asset['symbol'], name, self._asset_types[asset['type']], isin,
+            asset_id = JalDB().add_asset(asset['symbol'], name, self._asset_types[asset['type']], isin, expiry=expiry,
                                          data_source=source, reg_code=reg_code, country_code=country_code)
             if asset_id:
                 old_id, asset['id'] = asset['id'], -asset_id
@@ -286,13 +290,13 @@ class Statement:
                            f"{JalDB().get_asset_name(-transfer['asset'][1])} " + \
                            f"@{datetime.utcfromtimestamp(transfer['timestamp']).strftime('%d.%m.%Y')}\n" + \
                            g_tr('Statement', "Select account to withdraw from:")
-                    pair_account = transfer['account'][1]
+                    pair_account = -transfer['account'][1]
                 if transfer['account'][1] == 0:  # Withdrawal
                     text = g_tr('Statement', "Withdrawal of ") + f"{transfer['withdrawal']:.2f} " + \
                            f"{JalDB().get_asset_name(-transfer['asset'][0])} " + \
                            f"@{datetime.utcfromtimestamp(transfer['timestamp']).strftime('%d.%m.%Y')}\n" + \
                            g_tr('Statement', "Select account to deposit to:")
-                    pair_account = transfer['account'][0]
+                    pair_account = -transfer['account'][0]
                 pair_account = self.select_account(text, pair_account, self._last_selected_account)
                 if pair_account == 0:
                     raise Statement_ImportError(g_tr('Statement', "Account not selected"))
@@ -335,6 +339,8 @@ class Statement:
                 else:  # Dividend exists, only tax to be updated
                     JalDB().update_dividend_tax(-payment['id'], payment['tax'])
             elif payment['type'] == FOF.PAYMENT_INTEREST:
+                if 'number' not in payment:
+                    payment['number'] = ''
                 JalDB().add_dividend(DividendSubtype.BondInterest, payment['timestamp'], -payment['account'],
                                      -payment['asset'], payment['amount'], payment['description'], payment['number'],
                                      tax=tax)
@@ -369,9 +375,8 @@ class Statement:
     def select_account(self, text, account_id, recent_account_id=0):
         if "pytest" in sys.modules:
             return 1    # Always return 1st account if we are in testing mode
+        dialog = SelectAccountDialog(text, account_id, recent_account=recent_account_id)
+        if dialog.exec_() != QDialog.Accepted:
+            return 0
         else:
-            dialog = SelectAccountDialog(text, account_id, recent_account=recent_account_id)
-            if dialog.exec_() != QDialog.Accepted:
-                return 0
-            else:
-                return dialog.account_id
+            return dialog.account_id
