@@ -1,14 +1,14 @@
 import logging
 import re
-from datetime import datetime, timezone
+from datetime import datetime
 from itertools import groupby
-from lxml import etree
 
 from jal.constants import PredefinedCategory, DividendSubtype
 from jal.widgets.helpers import g_tr, ManipulateDate
 from jal.db.update import JalDB
 from jal.db.helpers import executeSQL, readSQLrecord
-from jal.data_import.statement import Statement, FOF
+from jal.data_import.statement import FOF, Statement_ImportError
+from jal.data_import.statement_xml import StatementXML
 
 
 # -----------------------------------------------------------------------------------------------------------------------
@@ -21,10 +21,6 @@ class IBKRCashOp:
     BondInterest = 5
 
 
-class IBKR_ParseError(Exception):
-    pass
-
-
 # -----------------------------------------------------------------------------------------------------------------------
 class IBKR_AssetType:
     NotSupported = -1
@@ -170,50 +166,113 @@ class IBKR_Account:
 
 # -----------------------------------------------------------------------------------------------------------------------
 # Class for Loading Interactive Brokers XML Flex report
-class StatementIBKR(Statement):
+class StatementIBKR(StatementXML):
+    statements_path = '/*/FlexStatement'
+    statement_tag = 'FlexStatement'
     CancelledFlag = 'Ca'
 
     def __init__(self):
         super().__init__()
-        self._data = {}
-
-    # -----------------------------------------------------------------------------------------------------------------------
-    # Helpers to get values from XML tag properties
-    # Convert attribute 'attr_name' value to string or return default value if attribute not found
-    @staticmethod
-    def attr_string(xml_element, attr_name, default_value):
-        if attr_name not in xml_element.attrib:
-            return default_value
-        return xml_element.attrib[attr_name]
-
-    # Convert attribute 'attr_name' value to float or return default value if attribute not found / not a number
-    @staticmethod
-    def attr_number(xml_element, attr_name, default_value):
-        if attr_name not in xml_element.attrib:
-            return default_value
-        try:
-            value = float(xml_element.attrib[attr_name])
-        except ValueError:
-            return None
-        return value
-
-    # Convert attribute 'attr_name' value from strings "YYYYMMDD:hhmmss' or "YYYYMMDD" to datetime object
-    # or return default value if attribute not found / has wrong format
-    @staticmethod
-    def attr_timestamp(xml_element, attr_name, default_value):
-        if attr_name not in xml_element.attrib:
-            return default_value
-        time_str = xml_element.attrib[attr_name]
-        try:
-            if len(time_str) == 15:  # YYYYMMDD;HHMMSS
-                return int(datetime.strptime(time_str, "%Y%m%d;%H%M%S").replace(tzinfo=timezone.utc).timestamp())
-            elif len(time_str) == 8:  # YYYYMMDD
-                return int(datetime.strptime(time_str, "%Y%m%d").replace(tzinfo=timezone.utc).timestamp())
-            else:
-                return default_value
-        except ValueError:
-            logging.error(g_tr('IBKR', "Unsupported date/time format: ") + f"{xml_element.attrib[attr_name]}")
-            return None
+        self.statement_name = g_tr('IBKR', "IBKR Flex-statement")
+        ibkr_loaders = {
+            IBKR_Currency: self.attr_currency,
+            IBKR_AssetType: self.attr_asset_type,
+            IBKR_Asset: self.attr_asset,
+            IBKR_Account: self.attr_account,
+            IBKR_CorpActionType: self.attr_corp_action_type
+        }
+        self.attr_loader.update(ibkr_loaders)
+        self._sections = {   # Order of load is important - accounts and assets should be loaded first
+            StatementXML.STATEMENT_ROOT: {'tag': self.statement_tag,
+                                          'level': '',
+                                          'values': [('accountId', 'account', str, None),
+                                                     ('fromDate', 'period_start', datetime, None),
+                                                     ('toDate', 'period_end', datetime, None)],
+                                          'loader': self.load_header
+                                          },
+            'CashReport': {'tag': 'CashReportCurrency',
+                           'level': 'Currency',
+                           'values': [('accountId', 'number', str, None),
+                                      ('currency', 'currency', IBKR_Currency, None),
+                                      ('startingCash', 'cash_begin', float, None),
+                                      ('endingCash', 'cash_end', float, None),    # -- this is planned
+                                      ('endingSettledCash', 'cash_end_settled', float, None)],   # -- this is now
+                           'loader': self.load_accounts},
+            'SecuritiesInfo': {'tag': 'SecurityInfo',
+                               'level': '',
+                               'values': [('symbol', 'symbol', str, None),
+                                          ('assetCategory', 'type', IBKR_AssetType, IBKR_AssetType.NotSupported),
+                                          ('description', 'name', str, None),
+                                          ('isin', 'isin', str, ''),
+                                          ('cusip', 'reg_code', str, ''),
+                                          ('expiry', 'expiry', datetime, 0),
+                                          ('maturity', 'maturity', datetime, 0),
+                                          ('listingExchange', 'exchange', str, '')],
+                               'loader': self.load_assets},
+            'Trades': {'tag': 'Trade',
+                       'level': 'EXECUTION',
+                       'values': [('assetCategory', 'type', IBKR_AssetType, IBKR_AssetType.NotSupported),
+                                  ('symbol', 'asset', IBKR_Asset, None),
+                                  ('accountId', 'account', IBKR_Account, None),
+                                  ('dateTime', 'timestamp', datetime, None),
+                                  ('settleDateTarget', 'settlement', datetime, 0),
+                                  ('tradePrice', 'price', float, None),
+                                  ('quantity', 'quantity', float, None),
+                                  ('proceeds', 'proceeds', float, None),
+                                  ('multiplier', 'multiplier', float, None),
+                                  ('ibCommission', 'fee', float, None),
+                                  ('tradeID', 'number', str, ''),
+                                  ('exchange', 'exchange', str, ''),
+                                  ('notes', 'notes', str, '')],
+                       'loader': self.load_ib_trades},
+            'OptionEAE': {'tag': 'OptionEAE',
+                          'level': '',
+                          'values': [('transactionType', 'operation', str, None),
+                                     ('symbol', 'asset', IBKR_Asset, None),
+                                     ('accountId', 'account', IBKR_Account, None),
+                                     ('date', 'timestamp', datetime, None),
+                                     ('tradePrice', 'price', float, None),
+                                     ('quantity', 'quantity', float, None),
+                                     ('multiplier', 'multiplier', float, None),
+                                     ('commisionsAndTax', 'fee', float, None),
+                                     ('tradeID', 'number', str, ''),
+                                     ('notes', 'notes', str, '')],
+                          'loader': self.load_options},
+            'CorporateActions': {'tag': 'CorporateAction',
+                                 'level': 'DETAIL',
+                                 'values': [('type', 'type', IBKR_CorpActionType, IBKR_CorpActionType.NotSupported),
+                                            ('accountId', 'account', IBKR_Account, None),
+                                            ('symbol', 'asset', IBKR_Asset, None),
+                                            (
+                                            'assetCategory', 'asset_type', IBKR_AssetType, IBKR_AssetType.NotSupported),
+                                            ('dateTime', 'timestamp', datetime, None),
+                                            ('transactionID', 'number', str, ''),
+                                            ('description', 'description', str, None),
+                                            ('quantity', 'quantity', float, None),
+                                            ('proceeds', 'proceeds', float, None),
+                                            ('code', 'code', str, '')],
+                                 'loader': self.load_corporate_actions},
+            'CashTransactions': {'tag': 'CashTransaction',
+                                 'level': 'DETAIL',
+                                 'values': [('type', 'type', str, None),
+                                            ('accountId', 'account', IBKR_Account, None),
+                                            ('symbol', 'asset', IBKR_Asset, 0),
+                                            ('currency', 'currency', IBKR_Currency, None),
+                                            ('dateTime', 'timestamp', datetime, None),
+                                            ('amount', 'amount', float, None),
+                                            ('tradeID', 'number', str, ''),
+                                            ('description', 'description', str, None)],
+                                 'loader': self.load_cash_transactions},
+            'TransactionTaxes': {'tag': 'TransactionTax',
+                                 'level': 'SUMMARY',
+                                 'values': [('accountId', 'account', IBKR_Account, None),
+                                            ('symbol', 'symbol', str, None),
+                                            ('date', 'timestamp', datetime, None),
+                                            ('taxAmount', 'amount', float, None),
+                                            ('description', 'description', str, None),
+                                            ('taxDescription', 'tax_description', str, None)],
+                                 'loader': self.load_taxes}
+        }
 
     # Convert attribute 'attr_name' value into json open-format asset type
     @staticmethod
@@ -284,12 +343,6 @@ class StatementIBKR(Statement):
         else:
             return account_id
 
-    # Removes all keys listed in extra_keys_list from operation_dict
-    def drop_extra_fields(self, operation_dict, extra_keys_list):
-        for key in extra_keys_list:
-            if key in operation_dict:
-                del operation_dict[key]
-
     def locate_asset(self, symbol, isin) -> int:
         candidates = [x for x in self._data[FOF.ASSETS] if 'isin' in x and x['isin'] == isin]
         if len(candidates) == 1:
@@ -303,161 +356,14 @@ class StatementIBKR(Statement):
         assets = [x for x in self._data[FOF.ASSETS] if 'id' in x and x['id'] == asset_id]
         if len(assets) != 1:
             return
-        assets[0]["country"] = country    # TODO: check if asset['country'] already has different value
-        return
+        assets[0]["country"] = country
 
-    def load(self, filename: str) -> None:
-        self._data = {
-            FOF.PERIOD: [None, None],
-            FOF.ACCOUNTS: [],
-            FOF.ASSETS: [],
-            FOF.TRADES: [],
-            FOF.TRANSFERS: [],
-            FOF.CORP_ACTIONS: [],
-            FOF.ASSET_PAYMENTS: [],
-            FOF.INCOME_SPENDING: []
-        }
-
-        section_loaders = {
-            'CashReport': self.load_accounts,  # Order of load is important - accounts and assets should be loaded first
-            'SecuritiesInfo': self.load_assets,
-            'Trades': self.load_ib_trades,
-            'OptionEAE': self.load_options,
-            'CorporateActions': self.load_corporate_actions,
-            'CashTransactions': self.load_cash_transactions,
-            'TransactionTaxes': self.load_taxes
-        }
-        try:
-            xml_root = etree.parse(filename)
-            for FlexStatements in xml_root.getroot():
-                for statement in FlexStatements:
-                    attr = statement.attrib
-                    self._data[FOF.PERIOD][0] = int(
-                        datetime.strptime(attr['fromDate'], "%Y%m%d").replace(tzinfo=timezone.utc).timestamp())
-                    self._data[FOF.PERIOD][1] = int(
-                        datetime.strptime(attr['toDate'], "%Y%m%d").replace(tzinfo=timezone.utc).timestamp())
-                    logging.info(g_tr('IBKR', "Load IB Flex-statement for account ") +
-                                 f"{attr['accountId']}: {attr['fromDate']} - {attr['toDate']}")
-                    for section in section_loaders:
-                        section_elements = statement.xpath(section)  # Actually should be list of 0 or 1 element
-                        if section_elements:
-                            section_data = self.get_ibkr_data(section_elements[0])
-                            if section_data is None:
-                                return
-                            section_loaders[section](section_data)
-        except Exception as e:
-            logging.error(g_tr('IBKR', "Failed to parse Interactive Brokers flex-report") + f": {e}",
-                          exc_info=True)
-            return
-        logging.info(g_tr('IBKR', "IB Flex-statement loaded successfully"))
-
-    def get_ibkr_data(self, section):
-        attr_loader = {
-            str: self.attr_string,
-            float: self.attr_number,
-            datetime: self.attr_timestamp,
-            IBKR_Currency: self.attr_currency,
-            IBKR_AssetType: self.attr_asset_type,
-            IBKR_Asset: self.attr_asset,
-            IBKR_Account: self.attr_account,
-            IBKR_CorpActionType: self.attr_corp_action_type
-        }
-
-        section_descriptions = {
-            'CashReport': {'tag': 'CashReportCurrency',
-                           'level': 'Currency',
-                           'values': [('accountId', 'number', str, None),
-                                      ('currency', 'currency', IBKR_Currency, None),
-                                      ('startingCash', 'cash_begin', float, None),
-                                      ('endingCash', 'cash_end', float, None),
-                                      ('endingSettledCash', 'cash_end_settled', float, None)]},
-            'SecuritiesInfo': {'tag': 'SecurityInfo',
-                               'level': '',
-                               'values': [('symbol', 'symbol', str, None),
-                                          ('assetCategory', 'type', IBKR_AssetType, IBKR_AssetType.NotSupported),
-                                          ('description', 'name', str, None),
-                                          ('isin', 'isin', str, ''),
-                                          ('cusip', 'reg_code', str, ''),
-                                          ('listingExchange', 'exchange', str, '')]},
-            'Trades': {'tag': 'Trade',
-                       'level': 'EXECUTION',
-                       'values': [('assetCategory', 'type', IBKR_AssetType, IBKR_AssetType.NotSupported),
-                                  ('symbol', 'asset', IBKR_Asset, None),
-                                  ('accountId', 'account', IBKR_Account, None),
-                                  ('dateTime', 'timestamp', datetime, None),
-                                  ('settleDateTarget', 'settlement', datetime, 0),
-                                  ('tradePrice', 'price', float, None),
-                                  ('quantity', 'quantity', float, None),
-                                  ('proceeds', 'proceeds', float, None),
-                                  ('multiplier', 'multiplier', float, None),
-                                  ('ibCommission', 'fee', float, None),
-                                  ('tradeID', 'number', str, ''),
-                                  ('exchange', 'exchange', str, ''),
-                                  ('notes', 'notes', str, '')]},
-            'OptionEAE': {'tag': 'OptionEAE',
-                          'level': '',
-                          'values': [('transactionType', 'operation', str, None),
-                                     ('symbol', 'asset', IBKR_Asset, None),
-                                     ('accountId', 'account', IBKR_Account, None),
-                                     ('date', 'timestamp', datetime, None),
-                                     ('tradePrice', 'price', float, None),
-                                     ('quantity', 'quantity', float, None),
-                                     ('multiplier', 'multiplier', float, None),
-                                     ('commisionsAndTax', 'fee', float, None),
-                                     ('tradeID', 'number', str, ''),
-                                     ('notes', 'notes', str, '')]},
-            'CorporateActions': {'tag': 'CorporateAction',
-                                 'level': 'DETAIL',
-                                 'values': [('type', 'type', IBKR_CorpActionType, IBKR_CorpActionType.NotSupported),
-                                            ('accountId', 'account', IBKR_Account, None),
-                                            ('symbol', 'asset', IBKR_Asset, None),
-                                            ('assetCategory', 'asset_type', IBKR_AssetType, IBKR_AssetType.NotSupported),
-                                            ('dateTime', 'timestamp', datetime, None),
-                                            ('transactionID', 'number', str, ''),
-                                            ('description', 'description', str, None),
-                                            ('quantity', 'quantity', float, None),
-                                            ('proceeds', 'proceeds', float, None),
-                                            ('code', 'code', str, '')]},
-            'CashTransactions': {'tag': 'CashTransaction',
-                                 'level': 'DETAIL',
-                                 'values': [('type', 'type', str, None),
-                                            ('accountId', 'account', IBKR_Account, None),
-                                            ('symbol', 'asset', IBKR_Asset, 0),
-                                            ('currency', 'currency', IBKR_Currency, None),
-                                            ('dateTime', 'timestamp', datetime, None),
-                                            ('amount', 'amount', float, None),
-                                            ('tradeID', 'number', str, ''),
-                                            ('description', 'description', str, None)]},
-            'TransactionTaxes': {'tag': 'TransactionTax',
-                                 'level': 'SUMMARY',
-                                 'values': [('accountId', 'account', IBKR_Account, None),
-                                            ('symbol', 'symbol', str, None),
-                                            ('date', 'timestamp', datetime, None),
-                                            ('taxAmount', 'amount', float, None),
-                                            ('description', 'description', str, None),
-                                            ('taxDescription', 'tax_description', str, None)]}
-        }
-
-        try:
-            tag = section_descriptions[section.tag]['tag']
-        except KeyError:
-            return []  # This section isn't used for import
-
-        data = []
-        for sample in section.xpath(tag):
-            tag_dictionary = {}
-            if section_descriptions[section.tag]['level']:  # Skip extra lines (SUMMARY, etc)
-                if self.attr_string(sample, 'levelOfDetail', '') != section_descriptions[section.tag]['level']:
-                    continue
-            for attr_name, key_name, attr_type, attr_default in section_descriptions[section.tag]['values']:
-                attr_value = attr_loader[attr_type](sample, attr_name, attr_default)
-                if attr_value is None:
-                    logging.error(
-                        g_tr('IBKR', "Failed to load attribute: ") + f"{attr_name} / {sample.attrib}")
-                    return None
-                tag_dictionary[key_name] = attr_value
-            data.append(tag_dictionary)
-        return data
+    def load_header(self, header):
+        self._data[FOF.PERIOD][0] = header['period_start']
+        self._data[FOF.PERIOD][1] = self._end_of_date(header['period_end'])
+        logging.info(g_tr('IBKR', "Load IB Flex-statement for account ") +
+                     f"{header['account']}: {datetime.utcfromtimestamp(header['period_start']).strftime('%Y-%m-%d')}" +
+                     f" - {datetime.utcfromtimestamp(header['period_end']).strftime('%Y-%m-%d')}")
 
     def load_accounts(self, balances):
         for i, balance in enumerate(sorted(balances, key=lambda x: x['currency'])):
@@ -475,6 +381,11 @@ class StatementIBKR(Statement):
             asset['symbol'] = asset['symbol'][:-len('.OLD')] if asset['symbol'].endswith('.OLD') else asset['symbol']
             if asset['exchange'] == '' or asset['exchange'] == 'VALUE':  # don't store 'VALUE' or empty exchange
                 asset.pop('exchange')
+            if asset['maturity']:
+                asset['expiry'] = asset['maturity']
+            if asset['expiry'] == 0:
+                asset.pop('expiry')
+            asset.pop('maturity')
             cnt += 1
             self._data[FOF.ASSETS].append(asset)
         logging.info(g_tr('IBKR', "Securities loaded: ") + f"{cnt} ({len(assets)})")
@@ -589,12 +500,10 @@ class StatementIBKR(Statement):
             if action['jal_processed']:
                 continue
             if action['type'] in action_loaders:
-                try:
-                    cnt += action_loaders[action['type']](action, parts_b)
-                except IBKR_ParseError as e:
-                    logging.error(e)
+                cnt += action_loaders[action['type']](action, parts_b)
             else:
-                logging.warning(g_tr('IBKR', "Corporate action type is not supported: ") + f"{action['type']}")
+                raise Statement_ImportError(
+                    g_tr('IBKR', "Corporate action type is not supported: ") + f"{action['type']}")
         logging.info(g_tr('IBKR', "Corporate actions loaded: ") + f"{cnt} ({len(actions)})")
 
     def load_merger(self, action, parts_b) -> int:
@@ -610,10 +519,10 @@ class StatementIBKR(Statement):
             if parts:
                 break
         if parts is None:
-            raise IBKR_ParseError(g_tr('IBKR', "Can't parse Merger description ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Can't parse Merger description ") + f"'{action}'")
         merger_a = parts.groupdict()
         if len(merger_a) != MergerPatterns[pattern_id].count("(?P<"):  # check expected number of matches
-            raise IBKR_ParseError(g_tr('IBKR', "Merger description miss some data ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Merger description miss some data ") + f"'{action}'")
         description_b = action['description'][:parts.span('symbol')[0]] + merger_a['symbol_old'] + ", "
         asset_b = self.locate_asset(merger_a['symbol_old'], merger_a['isin_old'])
         paired_record = list(filter(
@@ -622,7 +531,7 @@ class StatementIBKR(Statement):
                          and pair['type'] == action['type']
                          and pair['timestamp'] == action['timestamp'], parts_b))
         if len(paired_record) != 1:
-            raise IBKR_ParseError(g_tr('IBKR', "Can't find paired record for ") + f"{action}")
+            raise Statement_ImportError(g_tr('IBKR', "Can't find paired record for ") + f"{action}")
         if pattern_id == 1:
             self.add_merger_payment(action['timestamp'], action['account'], paired_record[0]['proceeds'],
                                     parts['currency'], action['description'])
@@ -650,13 +559,13 @@ class StatementIBKR(Statement):
 
         parts = re.match(SpinOffPattern, action['description'], re.IGNORECASE)
         if parts is None:
-            raise IBKR_ParseError(g_tr('IBKR', "Can't parse Spin-off description ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Can't parse Spin-off description ") + f"'{action}'")
         spinoff = parts.groupdict()
         if len(spinoff) != SpinOffPattern.count("(?P<"):  # check that expected number of groups was matched
-            raise IBKR_ParseError(g_tr('IBKR', "Spin-off description miss some data ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Spin-off description miss some data ") + f"'{action}'")
         asset_old = self.locate_asset(spinoff['symbol_old'], spinoff['isin_old'])
         if not asset_old:
-            raise IBKR_ParseError(g_tr('IBKR', "Spin-off initial asset not found ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Spin-off initial asset not found ") + f"'{action}'")
         qty_old = int(spinoff['Y']) * action['quantity'] / int(spinoff['X'])
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
         action['cost_basis'] = 0.0
@@ -671,10 +580,10 @@ class StatementIBKR(Statement):
 
         parts = re.match(SymbolChangePattern, action['description'], re.IGNORECASE)
         if parts is None:
-            raise IBKR_ParseError(g_tr('IBKR', "Can't parse Symbol Change description ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Can't parse Symbol Change description ") + f"'{action}'")
         isin_change = parts.groupdict()
         if len(isin_change) != SymbolChangePattern.count("(?P<"):  # check that expected number of groups was matched
-            raise IBKR_ParseError(g_tr('StatementLoader', "Spin-off description miss some data ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('StatementLoader', "Spin-off description miss some data ") + f"'{action}'")
         description_b = action['description'][:parts.span('symbol')[0]] + isin_change['symbol_old'] + ".OLD, "
         asset_b = self.locate_asset(isin_change['symbol_old'], isin_change['isin_old'])
         paired_record = list(filter(
@@ -683,7 +592,7 @@ class StatementIBKR(Statement):
                          and pair['type'] == action['type']
                          and pair['timestamp'] == action['timestamp'], parts_b))
         if len(paired_record) != 1:
-            raise IBKR_ParseError(g_tr('StatementLoader', "Can't find paired record for: ") + f"{action}")
+            raise Statement_ImportError(g_tr('StatementLoader', "Can't find paired record for: ") + f"{action}")
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
         action['cost_basis'] = 1.0
         action['asset'] = [paired_record[0]['asset'], action['asset']]
@@ -705,10 +614,10 @@ class StatementIBKR(Statement):
 
         parts = re.match(SplitPattern, action['description'], re.IGNORECASE)
         if parts is None:
-            raise IBKR_ParseError(g_tr('IBKR', "Can't parse Split description ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Can't parse Split description ") + f"'{action}'")
         split = parts.groupdict()
         if len(split) != SplitPattern.count("(?P<"):  # check that expected number of groups was matched
-            raise IBKR_ParseError(g_tr('IBKR', "Split description miss some data ") + f"'{action}'")
+            raise Statement_ImportError(g_tr('IBKR', "Split description miss some data ") + f"'{action}'")
         if parts['isin_old'] == parts['id']:  # Simple split without ISIN change
             qty_delta = action['quantity']
             if qty_delta >= 0:  # Forward split (X>Y)
@@ -725,15 +634,16 @@ class StatementIBKR(Statement):
             self._data[FOF.CORP_ACTIONS].append(action)
             return 1
         else:  # Split together with ISIN change and there should be 2nd record available
-            description_b = action['description'][:parts.span('symbol')[0]] + split['symbol_old'] + ".OLD, "
+            description_b = action['description'][:parts.span('symbol')[0]] + split['symbol_old']
             asset_b = self.locate_asset(split['symbol_old'], split['isin_old'])
             paired_record = list(filter(
                 lambda pair: pair['asset'] == asset_b
-                             and pair['description'].startswith(description_b)
+                             and (pair['description'].startswith(description_b + ", ")
+                                  or pair['description'].startswith(description_b + ".OLD, "))
                              and pair['type'] == action['type']
                              and pair['timestamp'] == action['timestamp'], parts_b))
             if len(paired_record) != 1:
-                raise IBKR_ParseError(g_tr('StatementLoader', "Can't find paired record for: ") + f"{action}")
+                raise Statement_ImportError(g_tr('StatementLoader', "Can't find paired record for: ") + f"{action}")
             action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
             action['cost_basis'] = 1.0
             action['asset'] = [paired_record[0]['asset'], action['asset']]
