@@ -4,8 +4,8 @@ from datetime import datetime, timezone
 
 from jal.widgets.helpers import g_tr
 from jal.constants import Setup, PredefinedCategory
-from jal.data_import.statement import FOF
-from jal.data_import.statement_xls import StatementXLS, XLS_ParseError
+from jal.data_import.statement import FOF, Statement_ImportError
+from jal.data_import.statement_xls import StatementXLS
 
 
 class StatementUKFU(StatementXLS):
@@ -239,10 +239,15 @@ class StatementUKFU(StatementXLS):
         operations = {
             'Ввод ДС': self.transfer_in,
             'Вывод ДС': self.transfer_out,
+            'Перевод ДС': self.transfer,
             'Налог': self.tax,
             'Доход по финансовым инструментам': self.dividend,
             'Погашение купона': self.interest,
-            'Погашение номинала': self.bond_repayment
+            'Погашение номинала': self.bond_repayment,
+            'Списано по сделке': None,   # These operations are results of trades
+            'Получено по сделке': None,
+            "Вариационная маржа": None,  # These are non-trade operations for derivatives
+            "Заблокировано средств ГО": None
         }
 
         row, headers = self.find_section_start("ДВИЖЕНИЕ ДЕНЕЖНЫХ СРЕДСТВ ЗА ОТЧЕТНЫЙ ПЕРИОД",  columns)
@@ -252,24 +257,43 @@ class StatementUKFU(StatementXLS):
         while row < self._statement.shape[0]:
             if self._statement[self.HeaderCol][row] == '' and self._statement[self.HeaderCol][row + 1] == '':
                 break
-
             operation = self._statement[headers['type']][row]
-            if operation not in operations:   # not supported type of operation
-                row += 1
-                continue
+            if operation not in operations:
+                raise Statement_ImportError(g_tr('UKFU', "Unsuppported cash transaction ") + f"'{operation}'")
             number = self._statement[headers['number']][row]
             timestamp = int(datetime.strptime(self._statement[headers['date']][row],
                                               "%d.%m.%Y").replace(tzinfo=timezone.utc).timestamp())
             amount = self._statement[headers['amount']][row]
             description = self._statement[headers['description']][row]
             account_id = self._find_account_id(self._account_number, self._statement[headers['currency']][row])
-
-            operations[operation](timestamp, number, account_id, amount, description)
-
+            if operations[operation] is not None:
+                operations[operation](timestamp, number, account_id, amount, description)
             cnt += 1
             row += 1
         logging.info(g_tr('Uralsib', "Cash operations loaded: ") + f"{cnt}")
 
+    def transfer(self, timestamp, number, account_id, amount, description):
+        TransferPattern = r"^Перевод ДС на с\/с (?P<account_to>[\w|\/]+) с с\/с (?P<account_from>[\w|\/]+)\..*$"
+        if amount < 0:  # there should be positive paired record
+            return
+        parts = re.match(TransferPattern, description, re.IGNORECASE)
+        if parts is None:
+            raise Statement_ImportError(g_tr('Uralsib', "Can't parse transfer description ") + f"'{description}'")
+        transfer = parts.groupdict()
+        if len(transfer) != TransferPattern.count("(?P<"):  # check that expected number of groups was matched
+            raise Statement_ImportError(g_tr('Uralsib', "Transfer description miss some data ") + f"'{description}'")
+        if transfer['account_from'] == transfer['account_to']:  # It is a technical record for incoming transfer
+            return
+        currency_id = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]['currency']
+        currency_name = [x for x in self._data[FOF.ASSETS] if x["id"] == currency_id][0]['symbol']
+        account_from = self._find_account_id(transfer['account_from'], currency_name)
+        account_to = self._find_account_id(transfer['account_to'], currency_name)
+        new_id = max([0] + [x['id'] for x in self._data[FOF.TRANSFERS]]) + 1
+        transfer = {"id": new_id, "account": [account_from, account_to, 0], "number": number,
+                    "asset": [currency_id, currency_id], "timestamp": timestamp,
+                    "withdrawal": amount, "deposit": amount, "fee": 0.0, "description": description}
+        self._data[FOF.TRANSFERS].append(transfer)
+
     def transfer_in(self, timestamp, number, account_id, amount, description):
         account = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]
         new_id = max([0] + [x['id'] for x in self._data[FOF.TRANSFERS]]) + 1
@@ -292,7 +316,7 @@ class StatementUKFU(StatementXLS):
 
         parts = re.match(DividendPattern, description, re.IGNORECASE)
         if parts is None:
-            raise XLS_ParseError(g_tr('UKFU', "Can't parse dividend description ") + f"'{description}'")
+            raise Statement_ImportError(g_tr('UKFU', "Can't parse dividend description ") + f"'{description}'")
         dividend_data = parts.groupdict()
         isin_match = re.match(ISINPattern, dividend_data['REG_CODE'])
         if isin_match:
@@ -308,7 +332,7 @@ class StatementUKFU(StatementXLS):
             try:
                 tax = float(dividend_data['TAX'])
             except ValueError:
-                raise XLS_ParseError(g_tr('UKFU', "Failed to convert dividend tax ") + f"'{description}'")
+                raise Statement_ImportError(g_tr('UKFU', "Failed to convert dividend tax ") + f"'{description}'")
         else:
             tax = 0
         amount = amount + tax   # Statement contains value after taxation while JAL stores value before tax
@@ -331,7 +355,7 @@ class StatementUKFU(StatementXLS):
         interest_data = parts.groupdict()
         asset_id = self._find_asset_id(symbol=interest_data['NAME'])
         if asset_id is None:
-            raise XLS_ParseError(g_tr('Uralsib', "Can't find asset for bond interest ") + f"'{description}'")
+            raise Statement_ImportError(g_tr('Uralsib', "Can't find asset for bond interest ") + f"'{description}'")
         new_id = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
         payment = {"id": new_id, "type": FOF.PAYMENT_INTEREST, "account": account_id, "timestamp": timestamp,
                    "number": number, "asset": asset_id, "amount": amount, "description": description}
@@ -347,7 +371,7 @@ class StatementUKFU(StatementXLS):
         interest_data = parts.groupdict()
         asset_id = self._find_asset_id(symbol=interest_data['NAME'])
         if not asset_id:
-            raise XLS_ParseError(g_tr('Uralsib', "Can't find asset for bond repayment ") + f"'{description}'")
+            raise Statement_ImportError(g_tr('Uralsib', "Can't find asset for bond repayment ") + f"'{description}'")
         match = [x for x in self.asset_withdrawal if x['asset'] == asset_id and x['timestamp'] == timestamp]
         if not match:
             logging.error(g_tr('Uralsib', "Can't find asset cancellation record for ") + f"'{description}'")
@@ -373,6 +397,9 @@ class StatementUKFU(StatementXLS):
 
     def load_broker_fee(self):
         header_row = self.find_row(self.SummaryHeader) + 1
+        if header_row < 0:
+            logging.warning(g_tr('Uralsib', "Can't get header to find fees"))
+            return
         header_found = False
         for i, row in self._statement.iterrows():
             if (not header_found) and (row[self.HeaderCol] == "Уплаченная комиссия, в том числе"):
@@ -390,6 +417,8 @@ class StatementUKFU(StatementXLS):
                         continue
                     if fee == 0:
                         continue
+                    if row[1] == 'комиссия торговой системы':  # Exchange fee is part of trades
+                        continue
                     account_id = self._find_account_id(self._account_number, self._statement[col][header_row])
                     new_id = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
                     fee = {"id": new_id, "timestamp": self._data[FOF.PERIOD][1], "account": account_id, "peer": 0,
