@@ -436,8 +436,14 @@ class Ledger(QObject):
                            "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id_new AND l.value > 0 "
                            "WHERE c.qty_new > 0 AND c.asset_id_new=:asset_id AND c.account_id=:account_id "
                            "AND s.id < :sid AND s.id > :last_sid "
+                           "UNION ALL "  # Next part is for spin-offs that where data might be in old part
+                           "SELECT s.id, ABS(c.qty), coalesce(l.value/c.qty, 0) AS price FROM corp_actions AS c "
+                           "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
+                           "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id AND l.value > 0 "
+                           "WHERE c.qty_new > 0 AND c.asset_id=:asset_id AND c.type=:spinoff AND c.account_id=:account_id "
+                           "AND s.id < :sid AND s.id > :last_sid "
                            ")ORDER BY id",
-                           [(":asset_id", asset_id), (":account_id", account_id),
+                           [(":asset_id", asset_id), (":account_id", account_id), (":spinoff", CorporateAction.SpinOff),
                             (":sid", seq_id), (":last_sid", last_sid)])
         while query.next():  # Perform match ("closure") of previous trades
             deal_sid, deal_qty, deal_price = readSQLrecord(query)  # deal_sid -> trade_sid
@@ -474,7 +480,7 @@ class Ledger(QObject):
         new_qty = self.current['price']
         new_value = processed_value
         if self.current['subtype'] == CorporateAction.SpinOff:
-            new_value = processed_value * self.current['fee_tax']
+            new_value = processed_value * (1 - self.current['fee_tax'])
             # Modify value for old asset
             self.appendTransaction(BookAccount.Assets, self.current['amount'], processed_value - new_value)
         # Create value for new asset
@@ -561,3 +567,14 @@ class Ledger(QObject):
             self.rebuild(from_timestamp=rebuild_dialog.getTimestamp(),
                          fast_and_dirty=rebuild_dialog.isFastAndDirty(), silent=False)
 
+    def get_asset_amount(self, timestamp, account_id, asset_id):
+        return readSQL("SELECT sum_amount FROM ledger_sums "
+                       "WHERE account_id=:account_id AND asset_id=:asset_id AND timestamp<=:timestamp "
+                       "AND (book_account=:money OR book_account=:assets)"
+                       "ORDER BY sid DESC LIMIT 1",
+                       [(":account_id", account_id), (":asset_id", asset_id), (":timestamp", timestamp),
+                        (":money", BookAccount.Money), (":assets", BookAccount.Assets)])
+
+    def reconcile(self, account_id, timestamp):
+        _ = executeSQL("UPDATE accounts SET reconciled_on=:timestamp WHERE id = :account_id",
+                       [(":timestamp", timestamp), (":account_id", account_id)])
