@@ -131,10 +131,10 @@ class DLSGDeclForeign(DLSGsection):
         income.description = description
         income.income_date = (datetime.utcfromtimestamp(timestamp).date() - date(1899, 12, 30)).days
         income.tax_payment_date = income.income_date
-        income.currency_code = currency[0]
-        income.currency_name = currency[1]
-        income.income_units = income.tax_units = currency[2]
-        income.income_rate = income.tax_rate = rate * currency[2]
+        income.currency_code = currency['code']
+        income.currency_name = currency['name']
+        income.income_units = income.tax_units = currency['multiplier']
+        income.income_rate = income.tax_rate = rate * currency['multiplier']
         income.income_currency = amount
         income.income_rub = amount_rub
         income.tax_currency = tax
@@ -156,48 +156,44 @@ class DLSGDeclForeign(DLSGsection):
 class DLSG:
     header_pattern = "DLSG            Decl(\d{4})0102FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
     header = "DLSG            Decl{:04d}0102FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
+    DIVIDEND_INCOME = 'dividend'
+    STOCK_INCOME = 'stock'
+    DERIVATIVE_INCOME = 'derivative'
 
     currencies = {
-        'USD': ('840', 'Доллар США', 100),
-        'EUR': ('978', 'Евро', 100),
-        'GBP': ('826', 'Фунт стерлингов', 100),
-        'CNY': ('156', 'Юань', 1000),
-        'CAD': ('124', 'Канадский доллар', 100),
-        'HKD': ('344', 'Гонконгский доллар', 1000),
-        'INR': ('356', 'Индийская рупия', 10000),
-        'JPY': ('392', 'Иена', 10000),
-        'SGD': ('702', 'Сингапурский доллар', 100),
-        'CHF': ('756', 'Швейцарский франк', 100),
-        'TRY': ('949', 'Турецкая лира', 100),
-        'BRL': ('986', 'Бразильский реал', 100)
-    }
-    codes = {
-        'dividend': ('14', '1010', 'Дивиденды', '0'),
-        'stock': ('13', '1530', '(01)Доходы от реализации ЦБ (обращ-ся на орг. рынке ЦБ)', '201'),
-        'derivative': ('13', '1532', '(06)Доходы по оп-циям с ПФИ (обращ-ся на орг. рынке ЦБ), баз. ак. по которым являются ЦБ', '206')
-    }
-    countries = {
-        'ru': '643',
-        'us': '840',
-        'ie': '372',
-        'ch': '756',
-        'fr': '250',
-        'ca': '124',
-        'se': '752',
-        'it': '380',
-        'es': '724',
-        'au': '036',
-        'at': '040',
-        'be': '056',
-        'gb': '826',
-        'de': '276',
-        'cn': '156',
-        'fi': '246',
-        'nl': '528',
-        'gr': '300',
-        'bm': '060',
-        'br': '076',
-        'je': '832'
+        'AUD': {'code': '036', 'name': 'Австралийский доллар', 'multiplier': 100},
+        'AZN': {'code': '944', 'name': 'Азербайджанский манат', 'multiplier': 100},
+        'GBP': {'code': '826', 'name': 'Фунт стерлингов', 'multiplier': 100},
+        'AMD': {'code': '051', 'name': 'Армянский драм', 'multiplier': 10000},
+        'BYN': {'code': '933', 'name': 'Белорусский рубль', 'multiplier': 100},
+        'BGN': {'code': '975', 'name': 'Болгарский лев', 'multiplier': 100},
+        'BRL': {'code': '986', 'name': 'Бразильский реал', 'multiplier': 100},
+        'HUF': {'code': '348', 'name': 'Форинт', 'multiplier': 10000},
+        'HKD': {'code': '344', 'name': 'Гонконгский доллар', 'multiplier': 1000},
+        'DKK': {'code': '208', 'name': 'Датская крона', 'multiplier': 100},
+        'USD': {'code': '840', 'name': 'Доллар США', 'multiplier': 100},
+        'EUR': {'code': '978', 'name': 'Евро', 'multiplier': 100},
+        'INR': {'code': '356', 'name': 'Индийская рупия', 'multiplier': 10000},
+        'KZT': {'code': '398', 'name': 'Тенге', 'multiplier': 10000},
+        'CAD': {'code': '124', 'name': 'Канадский доллар', 'multiplier': 100},
+        'KGS': {'code': '417', 'name': 'Сом', 'multiplier': 10000},
+        'CNY': {'code': '156', 'name': 'Юань', 'multiplier': 100},
+        'MDL': {'code': '498', 'name': 'Молдавский лей', 'multiplier': 1000},
+        'NOK': {'code': '578', 'name': 'Норвежская крона', 'multiplier': 1000},
+        'PLN': {'code': '985', 'name': 'Злотый', 'multiplier': 100},
+        'RON': {'code': '946', 'name': 'Румынский лей', 'multiplier': 100},
+        'SGD': {'code': '702', 'name': 'Сингапурский доллар', 'multiplier': 100},
+        'TJS': {'code': '972', 'name': 'Сомони', 'multiplier': 1000},
+        'TRY': {'code': '949', 'name': 'Турецкая лира', 'multiplier': 100},
+        'TMT': {'code': '934', 'name': 'Новый туркменский манат', 'multiplier': 100},
+        'UZS': {'code': '860', 'name': 'Узбекский сум', 'multiplier': 1000000},
+        'UAH': {'code': '980', 'name': 'Гривна', 'multiplier': 1000},
+        'CZK': {'code': '203', 'name': 'Чешская крона', 'multiplier': 1000},
+        'SEK': {'code': '752', 'name': 'Шведская крона', 'multiplier': 1000},
+        'CHF': {'code': '756', 'name': 'Швейцарский франк', 'multiplier': 100},
+        'ZAR': {'code': '710', 'name': 'Рэнд', 'multiplier': 1000},
+        'KRW': {'code': '410', 'name': 'Вона', 'multiplier': 100000},
+        'JPY': {'code': '392', 'name': 'Иена', 'multiplier': 10000}
     }
 
     def __init__(self, only_dividends=False):
@@ -207,60 +203,33 @@ class DLSG:
         self._sections = {}
         self._footer_len = 0        # if file ends with _footer_len 0x00 bytes
 
-    def add_dividend(self, country, description, timestamp, currency_name, amount, amount_rub, tax, tax_rub, rate):
-        foreign_section = self.get_section('DeclForeign')
-        if foreign_section is None:
-            return
-        try:
-            country_code, currency_code = self.get_country_currency(country, currency_name)
-        except ValueError:
-            logging.warning(g_tr('DLSG', "Dividend wasn't written to russian tax form"))
-            return
-        source = "Дивиденд от " + description
-        foreign_section.add_income(self.codes['dividend'], country_code, source, timestamp,
-                                   currency_code, amount, amount_rub, tax, tax_rub, rate)
+        self.codes = {
+            self.DIVIDEND_INCOME: ('14', '1010', 'Дивиденды', '0'),
+            self.STOCK_INCOME: ('13', '1530', '(01)Доходы от реализации ЦБ (обращ-ся на орг. рынке ЦБ)', '201'),
+            self.DERIVATIVE_INCOME: ('13', '1532', '(06)Доходы по оп-циям с ПФИ (обращ-ся на орг. рынке ЦБ), баз. ак. по которым являются ЦБ', '206')
+        }
 
-    def add_stock_profit(self, country, source, timestamp, currency_name, amount, income_rub, spending_rub, rate):
-        if self._only_dividends:
+    # Create an income record of given 'type' in tax declaration
+    # timestamp - date of income, country_code - 3-digit ISO country code, currency - 3-letter ISO currency code
+    # rate - currency rate for given date, note - description of the income, amount and tax are currency and rubles
+    def add_foreign_income(self, type, timestamp, country_code, currency,
+                           rate, amount, amount_rub, tax, tax_rub, note, spending_rub=0.0):
+        if type != DLSG.DIVIDEND_INCOME and self._only_dividends:
             return
         foreign_section = self.get_section('DeclForeign')
         if foreign_section is None:
             return
         try:
-            country_code, currency_code = self.get_country_currency(country, currency_name)
-        except ValueError:
-            logging.warning(g_tr('DLSG', "Operation with stock wasn't written to russian tax form"))
-            return
-        foreign_section.add_income(self.codes['stock'], country_code, source, timestamp,
-                                   currency_code, amount, income_rub, 0.0, 0.0, rate, deduction=spending_rub)
-
-    def add_derivative_profit(self, country, source, timestamp, currency_name, amount, income_rub, spending_rub, rate):
-        if self._only_dividends:
-            return
-        foreign_section = self.get_section('DeclForeign')
-        if foreign_section is None:
-            return
+            tax_codes = self.codes[type]
+        except KeyError:
+            raise(g_tr('DLSG', "Unknown income type for russian tax form: ") + type)
         try:
-            country_code, currency_code = self.get_country_currency(country, currency_name)
-        except ValueError:
-            logging.warning(g_tr('DLSG', "Operation with derivative wasn't written to russian tax form"))
+            currency_data = self.currencies[currency]
+        except KeyError:
+            logging.error(g_tr('DLSG', "Currency isn't supported for russian tax form: ") + currency)
             return
-        foreign_section.add_income(self.codes['derivative'], country_code, source, timestamp,
-                                   currency_code, amount, income_rub, 0.0, 0.0, rate, deduction=spending_rub)
-
-    def get_country_currency(self, country, currency_name):
-        try:
-            currency_code = self.currencies[currency_name]
-        except:
-            logging.error(g_tr('DLSG', "Currency code isn't known for russian tax form:") + f" {currency_name}")
-            raise ValueError
-        try:
-            country_code = self.countries[country]
-        except:
-            logging.error(g_tr('DLSG', "Country code isn't known for russian tax form (check account settings):") +
-                          f" {country}")
-            raise ValueError
-        return country_code, currency_code
+        foreign_section.add_income(tax_codes, country_code, note, timestamp,
+                                   currency_data, amount, amount_rub, tax, tax_rub, rate, deduction=spending_rub)
 
     def get_section(self, name):
         for section in self._sections:
