@@ -70,6 +70,9 @@ class TaxExportDialog(QDialog, Ui_TaxExportDlg):
     def getDslgOutFilename(self):
         return self.DlsgOutFileName.text()
 
+    def getBrokerAsIncomeName(self):
+        return self.IncomeSourceBroker.isChecked()
+
     def getDividendsOnly(self):
         return self.DividendsOnly.isChecked()
 
@@ -82,6 +85,7 @@ class TaxExportDialog(QDialog, Ui_TaxExportDlg):
     update_dlsg = Property(bool, fget=getDlsgState)
     dlsg_in_filename = Property(str, fget=getDslgInFilename)
     dlsg_out_filename = Property(str, fget=getDslgOutFilename)
+    dlsg_broker_as_income = Property(bool, fget=getBrokerAsIncomeName)
     dlsg_dividends_only = Property(bool, fget=getDividendsOnly)
     no_settelement = Property(bool, fget=getNoSettlement)
 
@@ -114,6 +118,7 @@ class TaxesRus:
         self.year_end = 0
         self.account_currency = ''
         self.broker_name = ''
+        self.broker_as_income = True
         self.use_settlement = True
         self.current_report = None
         self.data_start_row = 9
@@ -260,6 +265,7 @@ class TaxesRus:
         dialog = TaxExportDialog(parent)
         if dialog.exec_():
             self.use_settlement = not dialog.no_settelement
+            self.broker_as_income = dialog.dlsg_broker_as_income
             self.save2file(dialog.xls_filename, dialog.year, dialog.account, dlsg_update=dialog.update_dlsg,
                            dlsg_in=dialog.dlsg_in_filename, dlsg_out=dialog.dlsg_out_filename,
                            dlsg_dividends_only=dialog.dlsg_dividends_only)
@@ -419,7 +425,7 @@ class TaxesRus:
     def prepare_dividends(self):
         query = executeSQL("SELECT d.timestamp AS payment_date, s.name AS symbol, s.full_name AS full_name, "
                            "s.isin AS isin, d.amount AS amount, d.tax AS tax, q.quote AS rate , "
-                           "c.name AS country, c.code AS country_code, c.tax_treaty AS tax_treaty "
+                           "c.name AS country, c.iso_code AS country_iso, c.tax_treaty AS tax_treaty "
                            "FROM dividends AS d "
                            "LEFT JOIN assets AS s ON s.id = d.asset_id "
                            "LEFT JOIN accounts AS a ON d.account_id = a.id "
@@ -443,18 +449,22 @@ class TaxesRus:
                     dividend["tax2pay"] = 0
             self.add_report_row(row, dividend, even_odd=row)
 
-            if dividend["country_code"] == 'xx':
-                dividend["country_code"] = readSQL("SELECT code FROM accounts AS a LEFT JOIN countries AS c "
+            if dividend["country_iso"] == '000':
+                dividend["country_iso"] = readSQL("SELECT c.iso_code FROM accounts AS a LEFT JOIN countries AS c "
                                                    "ON c.id = a.country_id WHERE a.id=:account_id",
                                                    [(":account_id", self.account_id)])
                 logging.warning(g_tr('TaxesRus',
                                      "Account country will be used for 3-NDFL update as country is not set for asset ")
                                 + f"'{dividend['symbol']}'")
             if self.statement is not None:
-                self.statement.add_dividend(dividend["country_code"], f"{dividend['symbol']} ({dividend['full_name']})",
-                                            dividend['payment_date'], self.account_currency, dividend['amount'],
-                                            dividend['amount_rub'], dividend['tax'], dividend['tax_rub'],
-                                            dividend['rate'])
+                if self.broker_as_income:
+                    income_source = self.broker_name
+                else:
+                    income_source = f"Дивиденд от {dividend['symbol']} ({dividend['full_name']})"
+                self.statement.add_foreign_income(
+                    DLSG.DIVIDEND_INCOME, dividend['payment_date'], dividend["country_iso"], self.account_currency,
+                    dividend['rate'], dividend['amount'], dividend['amount_rub'], dividend['tax'], dividend['tax_rub'],
+                    income_source)
             row += 1
 
         self.reports_xls.add_totals_footer(self.current_sheet, start_row, row, [4, 5, 6, 7, 8, 9])
@@ -463,7 +473,7 @@ class TaxesRus:
     # -----------------------------------------------------------------------------------------------------------------------
     def prepare_stocks_and_etf(self):
         # Take all actions without conversion
-        query = executeSQL("SELECT s.name AS symbol, s.isin AS isin, d.qty AS qty, cc.code AS country_code, "
+        query = executeSQL("SELECT s.name AS symbol, s.isin AS isin, d.qty AS qty, cc.iso_code AS country_iso, "
                            "o.timestamp AS o_date, qo.quote AS o_rate, o.settlement AS os_date, o.number AS o_number, "
                            "qos.quote AS os_rate, o.price AS o_price, o.qty AS o_qty, o.fee AS o_fee, "
                            "c.timestamp AS c_date, qc.quote AS c_rate, c.settlement AS cs_date, c.number AS c_number, "
@@ -535,9 +545,13 @@ class TaxesRus:
                 if deal['qty'] < 0:  # short position - swap close/open dates/rates
                     deal['cs_date'] = deal['os_date']
                     deal['cs_rate'] = deal['os_rate']
-                self.statement.add_stock_profit(deal['country_code'], self.broker_name, deal['cs_date'],
-                                                self.account_currency, deal['income'], deal['income_rub'],
-                                                deal['spending_rub'], deal['cs_rate'])
+                if self.broker_as_income:
+                    income_source = self.broker_name
+                else:
+                    income_source = f"Доход от сделки с {deal['symbol']} ({deal['isin']})"
+                self.statement.add_foreign_income(
+                    DLSG.STOCK_INCOME, deal['cs_date'], deal['country_iso'], self.account_currency, deal['cs_rate'],
+                    deal['income'], deal['income_rub'], 0.0, 0.0, income_source, spending_rub=deal['spending_rub'])
             data_row = data_row + 1
         row = start_row + (data_row * 2)
 
@@ -547,7 +561,7 @@ class TaxesRus:
     # -----------------------------------------------------------------------------------------------------------------------
     def prepare_bonds(self):
         # First put all closed deals with bonds
-        query = executeSQL("SELECT s.name AS symbol, s.isin AS isin, d.qty AS qty, cc.code AS country_code, "
+        query = executeSQL("SELECT s.name AS symbol, s.isin AS isin, d.qty AS qty, cc.iso_code AS country_iso, "
                            "o.timestamp AS o_date, qo.quote AS o_rate, o.settlement AS os_date, o.number AS o_number, "
                            "qos.quote AS os_rate, o.price AS o_price, o.qty AS o_qty, o.fee AS o_fee, -oi.amount AS o_int, "
                            "c.timestamp AS c_date, qc.quote AS c_rate, c.settlement AS cs_date, c.number AS c_number, "
@@ -617,15 +631,19 @@ class TaxesRus:
                 if deal['qty'] < 0:  # short position - swap close/open dates/rates
                     deal['cs_date'] = deal['os_date']
                     deal['cs_rate'] = deal['os_rate']
-                self.statement.add_stock_profit(deal['country_code'], self.broker_name, deal['cs_date'],
-                                                self.account_currency, deal['income'], deal['income_rub'],
-                                                deal['spending_rub'], deal['cs_rate'])
+                if self.broker_as_income:
+                    income_source = self.broker_name
+                else:
+                    income_source = f"Доход от сделки с {deal['symbol']} ({deal['isin']})"
+                self.statement.add_foreign_income(
+                    DLSG.STOCK_INCOME, deal['cs_date'], deal['country_iso'], self.account_currency, deal['cs_rate'],
+                    deal['income'], deal['income_rub'], 0.0, 0.0, income_source, spending_rub=deal['spending_rub'])
             data_row = data_row + 1
         row = start_row + (data_row * 2)
 
         # Second - take all bond interest payments not linked with buy/sell transactions
         query = executeSQL("SELECT b.name AS symbol, b.isin AS isin, i.timestamp AS o_date, i.number AS number, "
-                           "i.amount AS interest, r.quote AS rate, cc.code AS country_code "
+                           "i.amount AS interest, r.quote AS rate, cc.iso_code AS country_iso "
                            "FROM dividends AS i "
                            "LEFT JOIN trades AS t ON i.account_id=1 AND i.number=t.number "
                            "AND i.timestamp=t.timestamp AND i.asset_id=t.asset_id "
@@ -647,9 +665,13 @@ class TaxesRus:
             self.add_report_row(row, interest, even_odd=data_row, alternative=2)
 
             if self.statement is not None:
-                self.statement.add_stock_profit(interest['country_code'], self.broker_name, interest['o_date'],
-                                                self.account_currency, interest['interest'], interest['interest_rub'],
-                                                0, interest['rate'])
+                if self.broker_as_income:
+                    income_source = self.broker_name
+                else:
+                    income_source = f"Купонный доход от {deal['symbol']} ({deal['isin']})"
+                self.statement.add_foreign_income(
+                    DLSG.STOCK_INCOME, interest['o_date'], interest['country_iso'], self.account_currency,
+                    interest['rate'], interest['interest'], interest['interest_rub'], 0.0, 0.0, income_source)
 
             data_row = data_row + 1
             row += 1
@@ -660,7 +682,7 @@ class TaxesRus:
     # -----------------------------------------------------------------------------------------------------------------------
     def prepare_derivatives(self):
         # Take all actions without conversion
-        query = executeSQL("SELECT s.name AS symbol, d.qty AS qty, cc.code AS country_code, "
+        query = executeSQL("SELECT s.name AS symbol, d.qty AS qty, cc.iso_code AS country_iso, "
                            "o.timestamp AS o_date, qo.quote AS o_rate, o.settlement AS os_date, o.number AS o_number, "
                            "qos.quote AS os_rate, o.price AS o_price, o.qty AS o_qty, o.fee AS o_fee, "
                            "c.timestamp AS c_date, qc.quote AS c_rate, c.settlement AS cs_date, c.number AS c_number, "
@@ -682,9 +704,10 @@ class TaxesRus:
                            "LEFT JOIN t_last_dates AS ldcs ON c.settlement=ldcs.ref_id "
                            "LEFT JOIN quotes AS qcs ON ldcs.timestamp=qcs.timestamp AND a.currency_id=qcs.asset_id "
                            "WHERE c.settlement>=:begin AND c.settlement<:end AND d.account_id=:account_id "
-                           "AND s.type_id == 6 "  # To select only derivatives
+                           "AND s.type_id = :derivative "
                            "ORDER BY s.name, o.timestamp, c.timestamp",
-                           [(":begin", self.year_begin), (":end", self.year_end), (":account_id", self.account_id)])
+                           [(":begin", self.year_begin), (":end", self.year_end),
+                            (":account_id", self.account_id), (":derivative", PredefinedAsset.Derivative)])
         start_row = self.data_start_row
         data_row = 0
         while query.next():
@@ -719,9 +742,14 @@ class TaxesRus:
                 if deal['qty'] < 0:  # short position - swap close/open dates/rates
                     deal['cs_date'] = deal['os_date']
                     deal['cs_rate'] = deal['os_rate']
-                self.statement.add_derivative_profit(deal['country_code'], self.broker_name, deal['cs_date'],
-                                                     self.account_currency, deal['income'], deal['income_rub'],
-                                                     deal['spending_rub'], deal['cs_rate'])
+                if self.broker_as_income:
+                    income_source = self.broker_name
+                else:
+                    income_source = f"Доход от сделки с {deal['symbol']}"
+                self.statement.add_foreign_income(
+                    DLSG.DERIVATIVE_INCOME, deal['cs_date'], deal['country_iso'], self.account_currency,
+                    deal['cs_rate'], deal['income'], deal['income_rub'], 0.0, 0.0, income_source,
+                    spending_rub=deal['spending_rub'])
             data_row = data_row + 1
         row = start_row + (data_row * 2)
 
