@@ -4,12 +4,13 @@ from dateutil import tz
 
 from PySide2.QtCore import Qt, Slot
 from PySide2.QtWidgets import QLabel, QDateTimeEdit, QPushButton, QTableView, QHeaderView
+from PySide2.QtGui import QFont
 from PySide2.QtSql import QSqlTableModel
 from jal.widgets.helpers import g_tr
 from jal.widgets.abstract_operation_details import AbstractOperationDetails
 from jal.widgets.reference_selector import AccountSelector, PeerSelector
 from jal.widgets.account_select import OptionalCurrencyComboBox
-from jal.db.helpers import db_connection, executeSQL
+from jal.db.helpers import db_connection, executeSQL, load_icon
 from jal.widgets.delegates import WidgetMapperDelegateBase, FloatDelegate, CategorySelectorDelegate, TagSelectorDelegate
 
 
@@ -51,18 +52,12 @@ class IncomeSpendingWidget(AbstractOperationDetails):
         self.peer_widget = PeerSelector(self)
         self.a_currency = OptionalCurrencyComboBox(self)
         self.a_currency.setText(g_tr("IncomeSpendingWidget", "Paid in foreign currency:"))
-        self.add_button = QPushButton(self)
-        self.add_button.setText(" +️ ")
-        self.add_button.setFont(self.bold_font)
-        self.add_button.setFixedWidth(self.add_button.fontMetrics().width("XXX"))
-        self.del_button = QPushButton(self)
-        self.del_button.setText(" — ️")
-        self.del_button.setFont(self.bold_font)
-        self.del_button.setFixedWidth(self.del_button.fontMetrics().width("XXX"))
-        self.copy_button = QPushButton(self)
-        self.copy_button.setText(" >> ️")
-        self.copy_button.setFont(self.bold_font)
-        self.copy_button.setFixedWidth(self.copy_button.fontMetrics().width("XXX"))
+        self.add_button = QPushButton(load_icon("add.png"), '', self)
+        self.add_button.setToolTip(g_tr("IncomeSpendingWidget", "Add detail"))
+        self.del_button = QPushButton(load_icon("remove.png"), '', self)
+        self.del_button.setToolTip(g_tr("IncomeSpendingWidget", "Remove detail"))
+        self.copy_button = QPushButton(load_icon("copy.png"), '', self)
+        self.copy_button.setToolTip(g_tr("IncomeSpendingWidget", "Copy detail"))
         self.details_table = QTableView(self)
         self.details_table.horizontalHeader().setFont(self.bold_font)
         self.details_table.setAlternatingRowColors(True)
@@ -93,6 +88,7 @@ class IncomeSpendingWidget(AbstractOperationDetails):
         self.layout.addItem(self.horizontalSpacer, 1, 8, 1, 1)
 
         self.add_button.clicked.connect(self.addChild)
+        self.copy_button.clicked.connect(self.copyChild)
         self.del_button.clicked.connect(self.delChild)
 
         super()._init_db("actions")
@@ -139,11 +135,29 @@ class IncomeSpendingWidget(AbstractOperationDetails):
             return
 
     @Slot()
-    def delChild(self):
+    def copyChild(self):
         idx = self.details_table.selectionModel().selection().indexes()
-        selected_row = idx[0].row()
-        self.details_model.removeRow(selected_row)
-        self.details_table.setRowHidden(selected_row, True)
+        src_record = self.details_model.record(idx[0].row())
+        new_record = self.details_model.record()
+        new_record.setValue("category_id", src_record.value("category_id"))
+        if src_record.value("tag_id"):
+            new_record.setValue("tag_id", src_record.value("tag_id"))
+        else:
+            new_record.setNull("tag_id")
+        new_record.setValue("amount", src_record.value("amount"))
+        new_record.setValue("amount_alt", src_record.value("amount_alt"))
+        new_record.setValue("note", src_record.value("note"))
+        if not self.details_model.insertRecord(-1, new_record):
+            logging.fatal(
+                g_tr('AbstractOperationDetails', "Failed to add new record: ") + self.details_model.lastError().text())
+            return
+
+    @Slot()
+    def delChild(self):
+        selection = self.details_table.selectionModel().selection().indexes()
+        for idx in selection:
+            self.details_model.removeRow(idx.row())
+            self.onDataChange(idx, idx, None)
 
     @Slot()
     def saveChanges(self):
@@ -166,6 +180,14 @@ class IncomeSpendingWidget(AbstractOperationDetails):
         self.dbUpdated.emit()
         return
 
+    @Slot()
+    def revertChanges(self):
+        self.model.revertAll()
+        self.details_model.revertAll()
+        self.modified = False
+        self.commit_button.setEnabled(False)
+        self.revert_button.setEnabled(False)
+
     def createNew(self, account_id=0):
         super().createNew(account_id)
         self.details_model.setFilter(f"action_details.pid = 0")
@@ -209,6 +231,7 @@ class DetailsModel(QSqlTableModel):
                          g_tr('DetailsModel', "Amount"),
                          g_tr('DetailsModel', "Note")]
         super().__init__(parent=parent_view, db=db)
+        self.deleted = []
         self.alt_currency = ''
         self._view = parent_view
 
@@ -220,6 +243,29 @@ class DetailsModel(QSqlTableModel):
                 return self._columns[section]
         return None
 
+    def removeRow(self, row, parent=None):
+        self.deleted.append(row)
+        super().removeRow(row)
+
+    def submitAll(self):
+        result = super().submitAll()
+        if result:
+            self.deleted = []
+        return result
+
+    def revertAll(self):
+        self.deleted = []
+        super().revertAll()
+
+    def data(self, index, role=Qt.DisplayRole):
+        if not index.isValid():
+            return None
+        if role == Qt.FontRole and (index.row() in self.deleted):
+            font = QFont()
+            font.setStrikeOut(True)
+            return font
+        return super().data(index, role)
+
     def configureView(self):
         self._view.setColumnHidden(0, True)
         self._view.setColumnHidden(1, True)
