@@ -16,7 +16,7 @@ from jal.db.backup_restore import JalBackup
 from jal.db.helpers import get_app_path, get_dbfilename, load_icon
 from jal.db.update import JalDB
 from jal.db.settings import JalSettings
-from jal.data_import.downloader import QuoteDownloader
+from jal.net.downloader import QuoteDownloader
 from jal.db.ledger import Ledger
 from jal.db.balances_model import BalancesModel
 from jal.db.holdings_model import HoldingsModel
@@ -26,6 +26,7 @@ from jal.data_import.statements import StatementLoader
 from jal.reports.taxes import TaxesRus
 from jal.data_import.slips import ImportSlipDialog
 from jal.db.tax_estimator import TaxEstimator
+from jal.widgets.price_chart import ChartWindow
 
 
 #-----------------------------------------------------------------------------------------------------------------------
@@ -160,7 +161,7 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
         self.actionCopy.triggered.connect(self.copyOperation)
         self.downloader.download_completed.connect(self.balances_model.update)
         self.downloader.download_completed.connect(self.holdings_model.update)
-        self.statements.load_completed.connect(self.ledger.rebuild)
+        self.statements.load_completed.connect(self.onStatementImport)
         self.ledger.updated.connect(self.balances_model.update)
         self.ledger.updated.connect(self.holdings_model.update)
 
@@ -218,7 +219,9 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
         about_box.setText(title)
         about = g_tr('MainWindow', "<p>More information, manuals and problem reports are at "
                                    "<a href=https://github.com/titov-vv/jal>github home page</a></p>"
-                                   "<p>Questions, comments, donations: <a href=mailto:jal@gmx.ru>jal@gmx.ru</a></p>")
+                                   "<p>Questions, comments, help or donations:</p>"
+                                   "<p><a href=mailto:jal@gmx.ru>jal@gmx.ru</a></p>"
+                                   "<p><a href=https://t.me/jal_support>Telegram</a></p>")
         about_box.setInformativeText(about)
         about_box.show()
 
@@ -276,12 +279,24 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
     def onHoldingsContextMenu(self, pos):
         index = self.HoldingsTableView.indexAt(pos)
         contextMenu = QMenu(self.HoldingsTableView)
+        actionShowChart = QAction(text=g_tr('Ledger', "Show Price Chart"), parent=self.HoldingsTableView)
+        actionShowChart.triggered.connect(
+            partial(self.showPriceChart, self.HoldingsTableView.viewport().mapToGlobal(pos), index))
+        contextMenu.addAction(actionShowChart)
         actionEstimateTax = QAction(text=g_tr('Ledger', "Estimate Russian Tax"), parent=self.HoldingsTableView)
         actionEstimateTax.triggered.connect(
             partial(self.estimateRussianTax, self.HoldingsTableView.viewport().mapToGlobal(pos), index))
         contextMenu.addAction(actionEstimateTax)
         contextMenu.popup(self.HoldingsTableView.viewport().mapToGlobal(pos))
 
+    @Slot()
+    def showPriceChart(self, position, index):
+        model = index.model()
+        account, asset, asset_qty = model.get_data_for_tax(index)
+        self.price_chart = ChartWindow(account, asset, asset_qty, position)
+        if self.price_chart.ready:
+            self.price_chart.open()
+
     @Slot()
     def estimateRussianTax(self, position, index):
         model = index.model()
@@ -331,7 +346,12 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
 
     @Slot()
     def reconcileAtCurrentOperation(self):
-        self.operations_model.reconcile_operation(self.current_index.row())
+        timestamp_idx = self.operations_model.index(self.current_index.row(), self.operations_model.COL_TIMESTAMP)
+        timestamp = self.operations_model.data(timestamp_idx, Qt.UserRole)
+        account_idx = self.operations_model.index(self.current_index.row(), self.operations_model.COL_ACCOUNT_ID)
+        account_id = self.operations_model.data(account_idx, Qt.UserRole)
+        self.ledger.reconcile(account_id, timestamp)
+        self.operations_model.refresh()
 
     @Slot()
     def deleteOperation(self):
@@ -383,3 +403,19 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
             QuotesListDialog().exec_()
         else:
             assert False
+
+    @Slot()
+    def onStatementImport(self, timestamp, totals):
+        self.ledger.rebuild()
+        for account_id in totals:
+            for asset_id in totals[account_id]:
+                amount = self.ledger.get_asset_amount(timestamp, account_id, asset_id)
+                if amount is not None:
+                    if abs(totals[account_id][asset_id] - amount) <= Setup.DISP_TOLERANCE:
+                        self.ledger.reconcile(account_id, timestamp)
+                        self.balances_model.update()   # Update required to display reconciled
+                    else:
+                        account = JalDB().get_account_name(account_id)
+                        asset = JalDB().get_asset_name(asset_id)
+                        logging.warning(g_tr('MainWindow', "Statement ending balance doesn't match: ") +
+                                        f"{account} / {asset} / {amount} <> {totals[account_id][asset_id]}")
